const THREE = require('three');

/**
 * 3D shape of line segment implemented with a cylinder.
 */
class LineSegment {

    constructor(start, end, radius, material) {

        this._material = material;

        this._intermediateGroup = new THREE.Group();
        this._intermediateGroup.rotateX(Math.PI/2);
        
        this._group = new THREE.Group();
        this._group.add(this._intermediateGroup);

        // Calculate length of the segment.
        let length = end.clone().sub(start).length();

        // Define the segment as a cylinder.
        this._geometry = new THREE.CylinderGeometry(radius, radius, length, 32);
        let cylinder = new THREE.Mesh(this._geometry, this._material);
        // As the segment group is rotated on the x axis, we translate the cylinder on the y axis.
        cylinder.position.y = 0.5 * length;
        this._intermediateGroup.add(cylinder);

        this._group.position.copy(start.clone());
        this._group.lookAt(end);
    }

    get object3D() {
        return this._group;
    }

    get material() {
        return this._material;
    }

    destroy() {
        // Geometry is disposed but not material as it is created in the class.
        this._geometry.dispose();
    }

}

// Export the constructor.
module.exports = LineSegment;