const THREE = require('three');

/** 
 * Scene manager for the Three.js scenes so that we can manage multiple div views.
 * The view manager used the scissor openGL feature to enable overlapping windows.
 */
class ViewManager {
    
    constructor() {
        this._views = [];
        this._stereo = new THREE.StereoCamera();
        this._stereo.aspect = 9/16;
    }

    addView(view) {
        this._views.push({ view: view });
    }
        
    init() {
        // Iterate the scenes to init.
        this._views.forEach((viewer) => {
            const view = viewer.view;
            view.init();
        });
    }

    render() {
        
        // Iterate the scenes to render.
        this._views.forEach((viewer) => {
            // Get the view.
            const view = viewer.view;
            
            // Call the scene render function.
            view.render();
        });
    }
}

const viewManager = new ViewManager();

// Export the view manager as object.
module.exports = viewManager;
