const viewManager = require('../../utils/view-manager');
const { requester, receiver } = require('../../main/context');
const { synchronize } = require('../../main/synchronizer');
const { mode } = require('../../server-state');
const PubSub = require('pubsub-js');

class AnglesPanelView {

    constructor() {
        
        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <div class='col-sm-2'>A1&#176;</div>
                <input type='text' class='col-sm-4' id='a1' disabled/>
                <div class='col-sm-2'>A4&#176;</div>
                <input type='text' class='col-sm-4' id='a4' disabled/>
            </div>
            <div class='row'>
                <div class='col-sm-2'>A2&#176;</div>
                <input type='text' class='col-sm-4' id='a2' disabled/>
                <div class='col-sm-2'>A5&#176;</div>
                <input type='text' class='col-sm-4' id='a5' disabled/>
            </div>
            <div class='row'>
                <div class='col-sm-2'>A3&#176;</div>
                <input type='text' class='col-sm-4' id='a3' disabled/>
                <div class='col-sm-2'>A6&#176;</div>
                <input type='text' class='col-sm-4' id='a6' disabled/>
            </div>
            <div class='row'>
                <div class='col-sm-2'>A3P&#176;</div>
                <input type='text' class='col-sm-4' id='a7' disabled/>
                <label class='col-sm-2'/>
                <label class='col-sm-4'/>
            </div>
            <div class='row'>
                <label class='col-sm-12'/>
            </div>
            <div class='row'>
            <div class='col-sm-2'>Status</div>
                <input type='text' class='col-sm-4 text-center' id='angleStatus' disabled/>
                <label class='col-sm-2'/>
                <button class='col-sm-4 btn btn-sm btn-primary' id='resync' type='button'>Resync</button>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Angles', function(container, state) {
            container.getElement().html(html);
        });
        
        viewManager.addView(this);
    }

    init() {

        // Register resync button event.
        $("#resync").click(() => {
            
            requester.request('resync', 0, (data) => {
                synchronize(data);
                PubSub.publish('reset-reciprocal-view-3d');
            });
        });

        // Background color change if a position is forbidden.
        receiver.on('update-status', (msg, value) => {
            this._setStatus(value);
        });

        // Hide the button if not in spy mode.
        if (mode.spyMode) {
            $("#resync").hide();
        }
    }

    /**
     * Set angles inputs value.
     */
    setInputs(value) {
        $('#a1').val(value.a1.toFixed(3));
        $('#a2').val(value.a2.toFixed(3));
        $('#a3').val(value.a3.toFixed(3));
        $('#a4').val(value.a4.toFixed(3));
        $('#a5').val(value.a5.toFixed(3));
        $('#a6').val(value.a6.toFixed(3));
        $('#a7').val(value.a3p.toFixed(3));
        this._setStatus(value);
    }

    /**
     * Get all angles inputs values and store
     * them into an object.
     */
    getInputs() {
        return {
            a1: $('#a1').val(),
            a2: $('#a2').val(),
            a3: $('#a3').val(),
            a4: $('#a4').val(),
            a5: $('#a5').val(),
            a6: $('#a6').val(),
            a3p: $('#a7').val()
        };
    }

    render() {
    }

    _setStatus(value) {
        if (value.status === 1) {
            $("#angleStatus").removeClass("angle-error");
            $("#angleStatus").addClass("angle-ok");
            $("#angleStatus").val('Ok');
        }
        else if (value.status !== 0) {
            $("#angleStatus").removeClass("angle-ok");
            $("#angleStatus").addClass("angle-error");
            $("#angleStatus").val('Error');
        }

        if (value.status !== 0) {
        
            for (let i = 1; i < 8; i++) {
                let id = '#a' + i;
                if (value.error === i) {
                    $(id).addClass("angle-error");
                }
                else {
                    $(id).removeClass("angle-error");
                }
            }
        }
    }
}

module.exports = AnglesPanelView;