const viewManager = require('../../utils/view-manager');
const PubSub = require('pubsub-js');
const grid = require('ag-grid');
const { config } = require('../../server-state');

class ConfigurationPanelView {

    constructor() {

        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <button class='col-sm-6 btn btn-sm btn-primary' id='saveCfg' type='button'>Save configuration</button>
                <button class='col-sm-6 btn btn-sm btn-primary' id='exportCfg' type='button'>Export configurations</button>
            </div>
            <div class='custom-file mt-1'>
                <input type='file' class='custom-file-input' id='cfgFile'>
                <label class='custom-file-label' for='cfgFile'>Import an existing configuration</label>
            </div>
            <div id='configList' class='ag-theme-fresh' style='height:150px; width:100%; padding-top:10px'/>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Configuration', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }

    init() {

        this._cols = [
            { headerName: "", field: "id", width: 100, enableSorting: false, width: 0, hide: true },
            { headerName: "Configuration name", field: "name", filter: "string", editable: true},
            { headerName: "Restore", field: "restore", filter: "number", editable: false, width: 100, cellRenderer: function(params)  {
                return `<button class='restoreConfig' data-id="${params.data.id}" style="width: 100%">Restore</button>`;
            }},
            { headerName: "Delete", field: "delete", filter: "number", editable: false, width: 80, cellRenderer: function(params)  {
                return `<button class='deleteConfig' data-node-id="${params.node.id}" data-id="${params.data.id}" style="width: 100%">X</button>`;
            }},
        ];

        this._grid = { columnDefs: this._cols, rowData: [],
            enableSorting: true, enableColResize: true, enableFilter: true,
            rowSelection: "multiple"
        };

        
        this._thegrid = new grid.Grid($("#configList")[0], this._grid);

        for(let id in config.configurations) {
            this.addConfig(config.configurations[id]);
        }

        $('#saveCfg').click(() => {
            PubSub.publish('save-conf', 0);

            $('#saveCfg').toggleClass('btn-primary btn-success');
            setTimeout(() => {
                $('#saveCfg').toggleClass('btn-primary btn-success');
            }, 1000);
        });

        $('#exportCfg').click(() => {
            PubSub.publish('export-conf', 0);

            $('#exportCfg').toggleClass('btn-primary btn-success');
            setTimeout(() => {
                $('#exportCfg').toggleClass('btn-primary btn-success');
            }, 1000);
        });

        $('#cfgFile').change((event) => {
            let files = event.target.files;

            if(files.length > 0) {
                var reader = new FileReader();

                reader.onload = (loadevent) => {
                    PubSub.publish('import-conf', loadevent.target.result);
                    event.target.value = '';
                };

                reader.readAsText(files[0]);
            }
        });
    }

    /**
     * Clear the config list.
     */
    clearConfig() {
        this._grid.api.setRowData([ ]);
    }

    /**
     * Add a config to the config list.
     */
    addConfig(config) {
        let newRow = { id: config.id, name: config.name };
        this._grid.api.updateRowData({ add: [newRow] });

        $(".deleteConfig").off().click((event) => {
            let id = parseInt($(event.target).attr('data-id'));
            let nodeId = parseInt($(event.target).attr('data-node-id'));

            let data = this._grid.api.getRowNode(nodeId).data;
            this._grid.api.updateRowData({ remove: [data] });

            PubSub.publish('del-conf', id);
        });

        $('.restoreConfig').off().click((event) => {
            let id = parseInt($(event.target).attr('data-id'));

            PubSub.publish('restore-conf', id);
        });
    }

    render() {
    }
}

module.exports = ConfigurationPanelView;