const viewManager = require('../../utils/view-manager');
const PubSub = require('pubsub-js');
const { plane } = require('../../server-state');

class PlanePanelView {

    constructor() {

        let html = `
        <div class='container' style='width:100%'>
            <div class='row'>
                <span class='col-sm-3'>A</span>
                <input type='text' class='col-sm-3' id='ax'/>
                <input type='text' class='col-sm-3' id='ay'/>
                <input type='text' class='col-sm-3' id='az'/>
            </div>
            <div class='row'>
                <span class='col-sm-3'>B</span>
                <input type='text' class='col-sm-3' id='bx'/>
                <input type='text' class='col-sm-3' id='by'/>
                <input type='text' class='col-sm-3' id='bz'/>
            </div>
        </div>`;

        // Using lambda function does not work here.
        myLayout.registerComponent('Scattering Plane', function(container, state) {
            container.getElement().html(html);
        });

        viewManager.addView(this);
    }

    init() {
        // Update plan parameters action
        $("#ax, #ay, #az, #bx, #by, #bz").keypress((event) => {
            if (event.which == 13 || event.keyCode == 13) {
                PubSub.publish('update-plane-parameters', this.getInputs());
            }
        });

        // Initialize inputs from model data.
        this.setInputs({
            ax: plane.ax, ay: plane.ay, az: plane.az,
            bx: plane.bx, by: plane.by, bz: plane.bz
        });
    }

    /**
     * Set plane planes parameters in panel.
     */
    setInputs(value) {
        $('#ax').val(value.ax.toFixed(3));
        $('#ay').val(value.ay.toFixed(3));
        $('#az').val(value.az.toFixed(3));
        $('#bx').val(value.bx.toFixed(3));
        $('#by').val(value.by.toFixed(3));
        $('#bz').val(value.bz.toFixed(3));
    }

    /**
     * Copy inputs values and store them into an object.
     */
    getInputs() {
        return {
            ax: parseFloat($('#ax').val()),
            ay: parseFloat($('#ay').val()),
            az: parseFloat($('#az').val()),
            bx: parseFloat($('#bx').val()),
            by: parseFloat($('#by').val()),
            bz: parseFloat($('#bz').val())
        };
    }

    render() {
    }
}

module.exports = PlanePanelView;