const Requester = require('../requester');

class WebRequester extends Requester {
    constructor(wss) {
        super();

        this._listeners = {};
        this._wss = wss;

        this._wss.addEventListener('message', (event) => {
            // Server ACK, call the callback.
            let packet = JSON.parse(event.data);

            if(this._listeners[packet.event]) {
                this._listeners[packet.event](packet.data);
            }
        });

        this._wss.addEventListener('open', (event) => {
            console.log('requester socket opened');
        });

        this._wss.addEventListener('close', (event) => {
            console.log('requester socket closed');
        });

        this._wss.addEventListener('error', (event) => {
            console.log('requester socket error');
        });
    }

    // The onReady function is used to ensure that the web socket is ready before sending a request.
    onReady(callback) {
        this._wss.onopen = callback;
    }

    request(req, data, callback) {
        
        // Register the listener for the event fired when the server finishes
        this._listeners['s-' + req] = callback; 

        let packet = {
            event: req,
            data: data
        };

        // Send the request to the server.
        this._wss.send(JSON.stringify(packet));
    }
}

module.exports = WebRequester;