const Model = require('./model');

var configCounter = 0;

class Configuration {
    constructor(id, name) {
        this._id = id;
        this._name = name;
        this._data = {};
    }

    get id() { return this._id; }

    get name() { return this._name; }
    set name(value) { this._name = value; }

    get data() { return this._data; }
    set data(value) { this._data = value; }

    toObject() {
        return {
            id: this._id,
            name: this._name,
            data: this._data
        };
    }
}

class ConfigurationModel extends Model {

    constructor(savables) {
        super();

        this._savables = savables;
        this._configurations = {};
    }

    get configurations() { return this._configurations; }

    /**
     * Convert a string into a configuration list.
     */
    import(data) {
        this._configurations = {};
        
        for(let cfgObj of data) {
            this._configurations[cfgObj.id] = new Configuration(cfgObj.id, cfgObj.name);
            this._configurations[cfgObj.id].data = cfgObj.data;
        }
    }

    /**
     * Convert all configuration into a string.
     */
    stringify() {
        return JSON.stringify(this.toObject());
    }

    /**
     * Delete a configuration
     */
    deleteConf(id) {
        delete this._configurations[id];
    }

    toObject() {
        let exportObj = [];

        for(let id in this._configurations) {
            let conf = this._configurations[id];
            exportObj.push({
                id: conf.id,
                name: conf.name,
                data: conf.data
            });
        }

        return exportObj;
    }

    /**
     * Save all savables into a configuration object.
     */
    save() {
        let id = configCounter++;
        let config = new Configuration(id, `Configuration ${id}`);

        for(let savable of this._savables) {
            savable.save(config.data);
        }

        this._configurations[id] = config;
        return config;
    }

    /**
     * Restore a configuration.
     */
    restore(id) {
        let config = this._configurations[id];

        for(let savable of this._savables) {
            savable.restore(config.data);
        }
    }
}

module.exports = ConfigurationModel;