const Model = require('./model');
const vEXPController = require('../vexp-controller');
const SymmetryConstraint = require('./samples/symmetry-constraint');


class SampleModel extends Model {
    constructor(as, bs, cs, aa, bb, cc, sg, sync) {
        super();

        this._as = as;
        this._bs = bs;
        this._cs = cs;
        this._aa = aa;
        this._bb = bb;
        this._cc = cc;
        this._sg = sg;
        this._fixedValues = [];

        this._symmetryConstraint = new SymmetryConstraint();

        this._sync = sync;
    }

    // Getters.
    get as() { return this._as; }
    get bs() { return this._bs; }
    get cs() { return this._cs; }
    get aa() { return this._aa; }
    get bb() { return this._bb; }
    get cc() { return this._cc; }
    get sg() { return this._sg; }

    // Setters.
    set as(value) { this._as = value; this._updateSample(); }
    set bs(value) { this._bs = value; this._updateSample(); }
    set cs(value) { this._cs = value; this._updateSample(); }
    set aa(value) { this._aa = value; this._updateSample(); }
    set bb(value) { this._bb = value; this._updateSample(); }
    set cc(value) { this._cc = value; this._updateSample(); }
    set sg(value) { this._sg = value; this._updateSample(); }

    set sync(value) {
        this._sync = value;
        vEXPController.syncSample = value;
    }

    /**
     * Update the vEXPController using sample model values.
     */
    _updateSample(sampleValues) {

        this._fixedValues = this._symmetryConstraint.updateRules(sampleValues);

        // Update the sample after symmetry constraint.
        this._as = sampleValues.as;
        this._bs = sampleValues.bs;
        this._cs = sampleValues.cs;
        this._aa = sampleValues.aa;
        this._bb = sampleValues.bb;
        this._cc = sampleValues.cc;
        this._sg = sampleValues.sg;
        this._sync = false;

        vEXPController.setSampleFromUI(this._as, this._bs, this._cs, this._aa, this._bb, this._cc, this._sg);
        vEXPController.syncSample = false;
    }

    /**
     * Set all sample parameters
     */
    setSample(as, bs, cs, aa, bb, cc, sg) {

        this._updateSample({ as, bs, cs, aa, bb, cc, sg });
    }

    toObject() {
        return {
            as: this._as,
            bs: this._bs,
            cs: this._cs,
            aa: this._aa,
            bb: this._bb,
            cc: this._cc,
            sg: this._sg,
            fixedValues: this._fixedValues,
            sync: this._sync
        };
    }

    /**
     * Save the model data into a config object (the saver object).
     */
    save(saver) {
        saver.sample = this.toObject(); 
    }

    /**
     * Restore the model state saved in the saver object (the configuration object).
     */
    restore(saver) {
        let sample = saver.sample;
        this.setSample(
            sample.as, sample.bs, sample.cs,
            sample.aa, sample.bb, sample.cc, sample.sg
        );
        this._sync = sample.sync;
    }
}

module.exports = SampleModel;