/**
 * Load native vexplib
 * @author Tobias Weber <tweber@ill.fr>
 * @date Mar-2019
 * @license see "LICENSE" file
 */


 class VExpLib
{
    constructor(options = {})
    {
        // library instance
        this._nativelib = null;

        // library name
        const nativeLibFile = "addonvexplib";

        // search paths
        const cwd = process.cwd();
        const nativeLibPaths = [
            // absolute paths
            cwd + "/",
            cwd + "/build/Release/",

            // relative paths
            "./",
            "../build/Release/",
            "../../build/Release/",
        ];

        // look for native lib in given paths
        for(let pathidx=0; pathidx<nativeLibPaths.length; ++pathidx) {
            const libfile = nativeLibPaths[pathidx] + nativeLibFile;
            console.log("Attempting to load native vexplib: " + libfile);

            try {
                this._nativelib = require(libfile); 

                if(this._nativelib !== null) {
                    console.log("Loaded native vexplib: " + libfile);
                    break;
                }    
            }
            catch(err) {
                this._nativelib = null;
            }
        }

        if(this._nativelib === null) {
            console.error("Native vexplib is not available!");
        }
    }


    get instance() {
        return this._nativelib;
    }
};


// ----------------------------------------------------------------------------
// export
module.exports = new VExpLib().instance;
// ----------------------------------------------------------------------------
